@@ -1,4 +1,3 @@
-var original = require('original')
 var parse = require('url').parse
 var events = require('events')
 var https = require('https')
@@ -15,6 +14,10 @@ var colon = 58
 var space = 32
 var lineFeed = 10
 var carriageReturn = 13
+// Beyond 256KB we could not observe any gain in performance
+var maxBufferAheadAllocation = 1024 * 256
+// Headers matching the pattern should be removed when redirecting to different origin
+var reUnsafeHeader = /^(cookie|authorization)$/i
 
 function hasBom (buf) {
   return bom.every(function (charCode, index) {
@@ -31,6 +34,8 @@ function hasBom (buf) {
  **/
 function EventSource (url, eventSourceInitDict) {
   var readyState = EventSource.CONNECTING
+  var headers = eventSourceInitDict && eventSourceInitDict.headers
+  var hasNewOrigin = false
   Object.defineProperty(this, 'readyState', {
     get: function () {
       return readyState
@@ -52,11 +57,12 @@ function EventSource (url, eventSourceInitDict) {
     readyState = EventSource.CONNECTING
     _emit('error', new Event('error', {message: message}))
 
-    // The url may have been changed by a temporary
-    // redirect. If that's the case, revert it now.
+    // The url may have been changed by a temporary redirect. If that's the case,
+    // revert it now, and flag that we are no longer pointing to a new origin
     if (reconnectUrl) {
       url = reconnectUrl
       reconnectUrl = null
+      hasNewOrigin = false
     }
     setTimeout(function () {
       if (readyState !== EventSource.CONNECTING || self.connectionInProgress) {
@@ -69,9 +75,9 @@ function EventSource (url, eventSourceInitDict) {
 
   var req
   var lastEventId = ''
-  if (eventSourceInitDict && eventSourceInitDict.headers && eventSourceInitDict.headers['Last-Event-ID']) {
-    lastEventId = eventSourceInitDict.headers['Last-Event-ID']
-    delete eventSourceInitDict.headers['Last-Event-ID']
+  if (headers && headers['Last-Event-ID']) {
+    lastEventId = headers['Last-Event-ID']
+    delete headers['Last-Event-ID']
   }
 
   var discardTrailingNewline = false
@@ -85,9 +91,10 @@ function EventSource (url, eventSourceInitDict) {
     var isSecure = options.protocol === 'https:'
     options.headers = { 'Cache-Control': 'no-cache', 'Accept': 'text/event-stream' }
     if (lastEventId) options.headers['Last-Event-ID'] = lastEventId
-    if (eventSourceInitDict && eventSourceInitDict.headers) {
-      for (var i in eventSourceInitDict.headers) {
-        var header = eventSourceInitDict.headers[i]
+    if (headers) {
+      var reqHeaders = hasNewOrigin ? removeUnsafeHeaders(headers) : headers
+      for (var i in reqHeaders) {
+        var header = reqHeaders[i]
         if (header) {
           options.headers[i] = header
         }
@@ -147,13 +154,17 @@ function EventSource (url, eventSourceInitDict) {
 
       // Handle HTTP redirects
       if (res.statusCode === 301 || res.statusCode === 302 || res.statusCode === 307) {
-        if (!res.headers.location) {
+        var location = res.headers.location
+        if (!location) {
           // Server sent redirect response without Location header.
           _emit('error', new Event('error', {status: res.statusCode, message: res.statusMessage}))
           return
         }
+        var prevOrigin = new URL(url).origin
+        var nextOrigin = new URL(location).origin
+        hasNewOrigin = prevOrigin !== nextOrigin
         if (res.statusCode === 307) reconnectUrl = url
-        url = res.headers.location
+        url = location
         process.nextTick(connect)
         return
       }
@@ -179,19 +190,36 @@ function EventSource (url, eventSourceInitDict) {
 
       // text/event-stream parser adapted from webkit's
       // Source/WebCore/page/EventSource.cpp
-      var isFirst = true
       var buf
+      var newBuffer
       var startingPos = 0
       var startingFieldLength = -1
+      var newBufferSize = 0
+      var bytesUsed = 0
+
       res.on('data', function (chunk) {
-        buf = buf ? Buffer.concat([buf, chunk]) : chunk
-        if (isFirst && hasBom(buf)) {
-          buf = buf.slice(bom.length)
+        if (!buf) {
+          buf = chunk
+          if (hasBom(buf)) {
+            buf = buf.slice(bom.length)
+          }
+          bytesUsed = buf.length
+        } else {
+          if (chunk.length > buf.length - bytesUsed) {
+            newBufferSize = (buf.length * 2) + chunk.length
+            if (newBufferSize > maxBufferAheadAllocation) {
+              newBufferSize = buf.length + chunk.length + maxBufferAheadAllocation
+            }
+            newBuffer = Buffer.alloc(newBufferSize)
+            buf.copy(newBuffer, 0, 0, bytesUsed)
+            buf = newBuffer
+          }
+          chunk.copy(buf, bytesUsed)
+          bytesUsed += chunk.length
         }
 
-        isFirst = false
         var pos = 0
-        var length = buf.length
+        var length = bytesUsed
 
         while (pos < length) {
           if (discardTrailingNewline) {
@@ -235,8 +263,10 @@ function EventSource (url, eventSourceInitDict) {
 
         if (pos === length) {
           buf = void 0
+          bytesUsed = 0
         } else if (pos > 0) {
-          buf = buf.slice(pos)
+          buf = buf.slice(pos, bytesUsed)
+          bytesUsed = buf.length
         }
       })
     })
@@ -272,7 +302,7 @@ function EventSource (url, eventSourceInitDict) {
         _emit(type, new MessageEvent(type, {
           data: data.slice(0, -1), // remove trailing newline
           lastEventId: lastEventId,
-          origin: original(url)
+          origin: new URL(url).origin
         }))
         data = ''
       }
@@ -443,3 +473,23 @@ function MessageEvent (type, eventInitDict) {
     }
   }
 }
+
+/**
+ * Returns a new object of headers that does not include any authorization and cookie headers
+ *
+ * @param {Object} headers An object of headers ({[headerName]: headerValue})
+ * @return {Object} a new object of headers
+ * @api private
+ */
+function removeUnsafeHeaders (headers) {
+  var safe = {}
+  for (var key in headers) {
+    if (reUnsafeHeader.test(key)) {
+      continue
+    }
+
+    safe[key] = headers[key]
+  }
+
+  return safe
+}
